%macro PharmaSUG_SA_enrich 
   (
    randam_seed    = ,     /*Specify an integer seed value for reproduce the identical simulation result. Value range: 1 <= randam_seed <= 2**31 -1 */ 
    sim_number     = ,     /*specify the simulation number*/
    sample_size    = ,     /*specify the sample size*/
	subgrp1_prop   = ,     /*Specify the proportion (in decimal format) of subgroup 1 in treatment arm. value range 0 < subgrp1_prop < 1*/
	distribution   = ,     /*Specify the data distribution. Either "Exponential" or "Weibull". Please put a double quote when invoking the macro*/
    exp_hr_h       = ,     /*For Exponential Distribution only. Leave it as blank if Weibull distribution is specified. Specify the hazard rate highest in treatment period. Value range: exp_hr_h > 0. */
	exp_hr_l       = ,     /*For Exponential Distribution only. Leave it as blank if Weibull distribution is specified. Specify the hazard rate lowest in treatment period. Value range: exp_hr_l > 0, and it should be less than exp_hr_h*/
	weibull_a      = ,     /*For Weibull Distribution only. Leave it as blank if Exponential distribution is specified. Specify the value for parameter a. Value range: weibull_a > 0, BUT weibull_a should NOT be equal to 1, otherwise, it's identical to exponential*/
	weibull_b      = ,     /*For Weibull Distribution only. Leave it as blank if Exponential distribution is specified. Specify the value for parameter b. Value range: weibull_b > 0*/
	wb_hr_ratio    = ,     /*For Weibull Distribution only. Leave it as blank if Exponential distribution is specified. Specify the ratio of Hazard Rate lowest vs. highest in the treatment period. Value range: 0 < weibull_dec_to < 1*/
    censor_prop    = ,     /*Specify the censor proportion in DECIMAL format. Value range: 0 < censor_prop(decimal) < 1*/
	subgrp1_type   = ,     /*Specify the hazard rate changing type for subgroup 1 of treatment arm (arm 2): increase, decrease, concave or convex. Please put the double quote when invoking the macro*/
    subgrp2_type   = ,     /*Specify the hazard rate changing type for subgroup 2 of treatment arm (arm 2): increase, decrease, concave or convex. It should be different from that in Subgroup 1. Please put the double quote when invoking the macro*/
	path           = ,     /*Specify the path to save the simulated dataset, power summary dataset, table and graph. Please DO NOT put a double quote.*/
    final_ds       = ,     /*Name the final simulation dateset, and it will be saved in the library of "PamaSUG"*/       
    weight         = ,     /*Specify the weight function you would like to explore. The cadidate includes: All, Fleming (Num1, Num2), logrank, LR, modpeto, peto, wilcoxon, Tarone. DO NOT PUT quota, and leave blank between functions if more than one are specified.
	                         For the details of the function, please refer: https://support.sas.com/documentation/cdl/en/statug/63962/HTML/default/viewer.htm#statug_lifetest_sect008.htm*/
    power_smry_ds  = ,     /*Name the power summary dateset, and it will be saved in the library of "PamaSUG"*/    
    report_name    = ,     /*Specify the final report name, which contains a table a graph for the power summary*/
);


%*********************************************************************************************************************;
%***Part I: Derive the simulation data********************************************************************************;
%*********************************************************************************************************************;

%*Derive the basic dataset*****;

%*create two local macro variables for censor time simulation based on related censor proportion and distribution parameter;
%let exp_hazard_rate_cen = (&censor_prop./(1 - &censor_prop.))*&exp_hr_h.;
%let wei_centime_b = &weibull_b.*((1-&censor_prop.)/&censor_prop.)**(1/(&weibull_a.));

%***delete duplicated dataset in work library;
proc datasets library=work kill;
run;
quit;

data sa_enrich_raw (drop = obs i);
	call streaminit(&randam_seed.);  
		do sim = 1 to &sim_number.;        
			ID = 0;               
			obs = 0;
			do i = 1 to &sample_size.;  
				obs + 1;
				ID + 1;
				sub_grp = .;

				%**generate the random values for event time and censor time from exponential or weibull distribution.;
		        %if %sysfunc(upcase(&distribution.)) = "EXPONENTIAL" %then %do;
				  event_time = Rand("EXPONENTIAL", 1/(&exp_hr_h.));  
                  censor_time = Rand("EXPONENTIAL", 1/(&exp_hazard_rate_cen.)); 
				%end;

			    %else %if %sysfunc(upcase(&distribution.)) = "WEIBULL" %then %do;
				  event_time = Rand("WEIBULL", &weibull_a., &weibull_b.);  
                  censor_time = Rand("WEIBULL", &weibull_a., &wei_centime_b.); ; 
				%end;
           
				%**generate the final survival time and censored indicator for arm 1(control arm);
				if obs <= %sysevalf(&sample_size.)/2 then do;  
					arm =1;
					if censor_time lt event_time then do;  
						censored = 1;
						survival_time = censor_time;
					end;
					else do;                
						censored = 0;
						survival_time = event_time;
					end ;

					if    0 <= obs <= %sysevalf(&sample_size.)*(0.5*%sysevalf(&subgrp1_prop.)) then sub_grp = 1; 
                    else if  %sysevalf(&sample_size.)*(0.5*%sysevalf(&subgrp1_prop.)) < obs <= %sysevalf(&sample_size.)*0.5 then sub_grp = 2; 

				end ;

				%**blank the survival time and censored indicator for arm 2(treatment arm), create observation indicator for two subgroups;
				else if obs > %sysevalf(&sample_size.)/2 then do;  
					arm = 2; 
					survival_time = .;  
					censored      = .;  

                if %sysevalf(&sample_size.)/2 < obs <= %sysevalf(&sample_size.)*(0.5 + 0.5*%sysevalf(&subgrp1_prop.)) then sub_grp = 1; 
                else if obs > %sysevalf(&sample_size.)*(0.5 + 0.5*%sysevalf(&subgrp1_prop.))  then sub_grp = 2; 

               end;
        
			output;	
		end;
	  end;
run;


ods listing close;  
ods html close; 

%*Get the P20, P40, P60, P80 percentile of event time for arm 2;
ods output Summary = trt2_pertle;
proc means data = sa_enrich_raw p20 p40 p60 p80;
 where arm = 2;
 class sim sub_grp; 
 var event_time;
run;

data trt2_pertle_2; 
 set trt2_pertle;
 arm = 2;
 drop NOBS;
run;

%**merge the percentile event time values from arm 2 with the whole dataset;
proc sort data = trt2_pertle_2;
  by sim arm sub_grp; 
run;

proc sort data = sa_enrich_raw;
  by sim arm sub_grp; 
run;

data prefnl_sa_enrich;
 merge trt2_pertle_2 sa_enrich_raw;
  by sim arm sub_grp; 
  if arm = 2 then time_percentile_grp = 1*(event_time lt event_time_P20)+
                          2*(event_time_P20 le event_time lt event_time_P40)+
                          3*(event_time_P40 le event_time lt event_time_P60)+
                          4*(event_time_P60 le event_time lt event_time_P80)+
                          5*(event_time ge event_time_P80);

run;


%*create final dataset: update the event time and censor time for arm 2 based on the HR changing type**;
data final_sa_enrich;
  set prefnl_sa_enrich;

  if arm = 2 and sub_grp = 1 then do;

	 %if %sysfunc(upcase(&distribution.)) = "EXPONENTIAL" %then %do;

	       %if %sysfunc(lowcase(&subgrp1_type.)) = "decrease" %then %do; 
	            event_time_2 = event_time * &exp_hr_h./(&exp_hr_h. - (&exp_hr_h.-&exp_hr_l.)*time_percentile_grp/5); 
	            censor_time_2 = censor_time * &exp_hr_h./(&exp_hr_h. - (&exp_hr_h.-&exp_hr_l.)*time_percentile_grp/5);
		   %end;

	       %else %if %sysfunc(lowcase(&subgrp1_type.)) = "increase" %then %do; 
	            event_time_2 = event_time * &exp_hr_h./(&exp_hr_h. - (&exp_hr_h.-&exp_hr_l.)*(6-time_percentile_grp)/5); 
	            censor_time_2 = censor_time * &exp_hr_h./(&exp_hr_h. - (&exp_hr_h.-&exp_hr_l.)*(6-time_percentile_grp)/5); 
	       %end;

		   %else %if %sysfunc(lowcase(&subgrp1_type.)) = "concave" %then %do; 
	           event_time_2 = event_time * &exp_hr_h./(&exp_hr_h. - (&exp_hr_h.-&exp_hr_l.)*abs(3-time_percentile_grp)/2); 
	           censor_time_2 = censor_time * &exp_hr_h./(&exp_hr_h. - (&exp_hr_h.-&exp_hr_l.)*abs(3-time_percentile_grp)/2); 
	       %end;

	 
		   %else %if %sysfunc(lowcase(&subgrp1_type.)) = "convex" %then %do; 
	            event_time_2 = event_time * &exp_hr_h./(&exp_hr_h. - (&exp_hr_h.-&exp_hr_l.)*abs(2-abs(3-time_percentile_grp))/2); 
	            censor_time_2 = censor_time * &exp_hr_h./(&exp_hr_h. - (&exp_hr_h.-&exp_hr_l.)*abs(2-abs(3-time_percentile_grp))/2); 
	       %end;
	   %end;

	  %else %if %sysfunc(upcase(&distribution.)) = "WEIBULL" %then %do;

	       %if %sysfunc(lowcase(&subgrp1_type.)) = "decrease" %then %do; 
	            event_time_2 = event_time * (1-(1-&wb_hr_ratio.)*time_percentile_grp/5)**(1/(&weibull_a.-1)); 
	            censor_time_2 = censor_time * (1-(1-&wb_hr_ratio.)*time_percentile_grp/5)**(1/(&weibull_a.-1));
		   %end;

	       %else %if %sysfunc(lowcase(&subgrp1_type.)) = "increase" %then %do; 
	            event_time_2 = event_time * (1-(1-&wb_hr_ratio.)*(6-time_percentile_grp)/5)**(1/(&weibull_a.-1)); 
	            censor_time_2 = censor_time * (1-(1-&wb_hr_ratio.)*(6-time_percentile_grp)/5)**(1/(&weibull_a.-1));

	       %end;

		   %else %if %sysfunc(lowcase(&subgrp1_type.)) = "concave" %then %do; 
	           event_time_2 = event_time *(1-(1-&wb_hr_ratio.)*abs(time_percentile_grp-3)/2)**(1/(&weibull_a.-1)) ; 
	           censor_time_2 = censor_time *(1-(1-&wb_hr_ratio.)*abs(time_percentile_grp-3)/2)**(1/(&weibull_a.-1)); 
	       %end;

	 
		   %else %if %sysfunc(lowcase(&subgrp1_type.)) = "convex" %then %do; 
	            event_time_2 = event_time *(1-(1-&wb_hr_ratio.)*abs(2-abs(time_percentile_grp-3))/2)**(1/(&weibull_a.-1)) ; 
	            censor_time_2 = censor_time *(1-(1-&wb_hr_ratio.)*abs(2-abs(time_percentile_grp-3))/2)**(1/(&weibull_a.-1)) ; 
	       %end;

	   %end;

   end;

   else if arm = 2 and sub_grp = 2 then do;

	  %if %sysfunc(upcase(&distribution.)) = "EXPONENTIAL" %then %do;

	      %if %sysfunc(lowcase(&subgrp2_type.)) = "decrease" %then %do; 
	          event_time_2 = event_time * &exp_hr_h./(&exp_hr_h. - (&exp_hr_h.-&exp_hr_l.)*time_percentile_grp/5); 
	          censor_time_2 = censor_time * &exp_hr_h./(&exp_hr_h. - (&exp_hr_h.-&exp_hr_l.)*time_percentile_grp/5);
		   %end;

	       %else %if %sysfunc(lowcase(&subgrp2_type.)) = "increase" %then %do; 
	          event_time_2 = event_time * &exp_hr_h./(&exp_hr_h. - (&exp_hr_h.-&exp_hr_l.)*(6-time_percentile_grp)/5); 
	          censor_time_2 = censor_time * &exp_hr_h./(&exp_hr_h. - (&exp_hr_h.-&exp_hr_l.)*(6-time_percentile_grp)/5); 
	       %end;

		   %else %if %sysfunc(lowcase(&subgrp2_type.)) = "concave" %then %do; 
	          event_time_2 = event_time * &exp_hr_h./(&exp_hr_h. - (&exp_hr_h.-&exp_hr_l.)*abs(3-time_percentile_grp)/2); 
	          censor_time_2 = censor_time * &exp_hr_h./(&exp_hr_h. - (&exp_hr_h.-&exp_hr_l.)*abs(3-time_percentile_grp)/2); 
	       %end;

	 
		   %else %if %sysfunc(lowcase(&subgrp2_type.)) = "convex" %then %do; 
	          event_time_2 = event_time * &exp_hr_h./(&exp_hr_h. - (&exp_hr_h.-&exp_hr_l.)*abs(2-abs(3-time_percentile_grp))/2); 
	          censor_time_2 = censor_time * &exp_hr_h./(&exp_hr_h. - (&exp_hr_h.-&exp_hr_l.)*abs(2-abs(3-time_percentile_grp))/2); 
	       %end;

	   %end;
	 

	   %else %if %sysfunc(upcase(&distribution.)) = "WEIBULL" %then %do;

	       %if %sysfunc(lowcase(&subgrp2_type.)) = "decrease" %then %do; 
	            event_time_2 = event_time * (1-(1-&wb_hr_ratio.)*time_percentile_grp/5)**(1/(&weibull_a.-1)); 
	            censor_time_2 = censor_time * (1-(1-&wb_hr_ratio.)*time_percentile_grp/5)**(1/(&weibull_a.-1));
		   %end;

	       %else %if %sysfunc(lowcase(&subgrp2_type.)) = "increase" %then %do; 
	            event_time_2 = event_time * (1-(1-&wb_hr_ratio.)*(6-time_percentile_grp)/5)**(1/(&weibull_a.-1)); 
	            censor_time_2 = censor_time * (1-(1-&wb_hr_ratio.)*(6-time_percentile_grp)/5)**(1/(&weibull_a.-1));

	       %end;

		   %else %if %sysfunc(lowcase(&subgrp2_type.)) = "concave" %then %do; 
	           event_time_2 = event_time *(1-(1-&wb_hr_ratio.)*abs(time_percentile_grp-3)/2)**(1/(&weibull_a.-1)) ; 
	           censor_time_2 = censor_time *(1-(1-&wb_hr_ratio.)*abs(time_percentile_grp-3)/2)**(1/(&weibull_a.-1)); 
	       %end;

	 
		  %else  %if %sysfunc(lowcase(&subgrp2_type.)) = "convex" %then %do; 
	            event_time_2 = event_time *(1-(1-&wb_hr_ratio.)*abs(2-abs(time_percentile_grp-3))/2)**(1/(&weibull_a.-1)) ; 
	            censor_time_2 = censor_time *(1-(1-&wb_hr_ratio.)*abs(2-abs(time_percentile_grp-3))/2)**(1/(&weibull_a.-1)) ; 
	       %end;

	   %end;

	 end;

     if arm = 2 and sub_grp in (1 2) then do;

            if censor_time_2 lt event_time_2 then do; 
				censored = 1;
				survival_time = censor_time_2;
			end;

			else do;                 
				censored = 0;
				survival_time = event_time_2;
			end ;
    end;

run;

***set up a permanent library to save the simulated dataset, power summary dataset, table and graph;
libname PamaSUG "&path.";

 data PamaSUG.&final_ds.;
   set final_sa_enrich (drop = event_time_: censor_time_2 time_percentile_grp);
 run;

%*********************************************************************************************************************;
%***Part I Done: Derive the simulation data***************************************************************************;
%*********************************************************************************************************************;


%*********************************************************************************************************************;
%***Part II: Execute the log-rank test********************************************************************************;
%*********************************************************************************************************************;

ods output HomTests = pre_power;
proc lifetest data = PamaSUG.&final_ds.;
  by sim;
  time survival_time * censored (1); 
  strata /test =&weight. group = arm;
run;

data power;
 set pre_power;
 if ProbChiSq le 0.05 then identify = 1;
 else if ProbChiSq gt 0.05 then identify = 0;
run;

%*********************************************************************************************************************;
%***Part II Done: Execute the log-rank test***************************************************************************;
%*********************************************************************************************************************;


%*********************************************************************************************************************;
%***Part III: create table and graph for log-rank test with different weight functions*********************************;
%*********************************************************************************************************************;

ods output CrossTabFreqs = pre_final;
proc freq data = power;
 table Test * identify/nocol nopercent;
run;

data PamaSUG.&power_smry_ds.;
  set pre_final;
  where Test ne " " and identify = 1 ;
  RowPercent = RowPercent /100;
  sample_size = &sample_size.;
  simulation = &sim_number.;
  subgrp_prop = "&subgrp1_prop. vs. %sysevalf(1-&subgrp1_prop.)";
  censor_prop = &censor_prop.;
  subgrp1_type = "Hazard Rate "||&subgrp1_type.;
  subgrp2_type = "Hazard Rate "||&subgrp2_type.;
  Distribution =  &distribution.;

  %if %sysfunc(upcase(&distribution.)) = "EXPONENTIAL" %then %do;
    Parameter = "Hazard rate at highest: &exp_hr_h.. Hazard rate at lowest: &exp_hr_l.";
  %end;

  %else %if %sysfunc(upcase(&distribution.)) = "WEIBULL" %then %do;
    Parameter = "Weibull parameter a: &weibull_a., Weibull parameter b: &weibull_b.. the ratio of the hazard rate at lowest vs. highest is  &wb_hr_ratio.";
  %end;

  label
  RowPercent = "Empirical Power"
  sample_size = "Sample Size"
  simulation =  "Simulation Number"
  subgrp_prop = "Subgroup Ratio"
  censor_prop = "Censor Proportion"
  subgrp1_type = "Subgroup 1 Hazard Rate Change type"
  subgrp2_type = "Subgroup 2 Hazard Rate Change type"
  Distribution = "Survival Data Distribution Assumption"
  Parameter = "Distribution Parameter Information";
         
      
  drop Table _TYPE_ _TABLE_ Missing;

run;

options orientation=portrait;
ods pdf file = "&path.\&report_name..pdf" startpage = never;
title "Summary of empirical power profile for log-rank test with different weight functions";
Proc print data = PamaSUG.&power_smry_ds. (drop = Frequency identify) noobs label;
  run;

 title " ";
 ods graphics/ noborder;
 proc sgplot data = PamaSUG.&power_smry_ds.;
   vbar Test/response = RowPercent barwidth = 0.3;
   yaxis max = 1;
   label Test = "Weight Function";
 run;

 ods pdf close;

%mend;


****Macro invocation: example 1: 
	Simulation number: 1000
	Sample size: 100
	Two arms, and randomization ratio: 1 vs. 1
	Proportion of Subgroups: 0.5 vs. 0.5
	Survival data distribution assumption: Exponential
	Hazard rate highest in treatment period: 4
	Hazard rate lowest in treatment period: 1
	Censor Proportion: 0.2
	Subgroup 1 in arm 2 hazard rate changing type: Decrease
	Subgroup 2 in arm 2 hazard rate changing type: Increase
	Weight functions for log-rank test: logrank fleming(0,1) wilcoxon
;

%PharmaSUG_SA_enrich
(randam_seed  = 2046, 
sim_number    = 1000, 
sample_size   = 100, 
subgrp1_prop  = 0.5, 
distribution  = "Exponential", 
exp_hr_h      = 4, 
exp_hr_l      = 1, 
weibull_a     = , 
weibull_b     = , 
wb_hr_ratio   = , 
censor_prop   = 0.2, 
subgrp1_type  = "Decrease", 
subgrp2_type  = "Increase",
path          = C:\Users\Chwzhang2015\Desktop\PharmaSUG 2019 June 15-20\!!!Code & dataset\PharmaSUG,
final_ds      = exp_1, 
weight        = logrank fleming(0,1) Wilcoxon,
power_smry_ds = exp_smry_1,
report_name   = exp_smry_1_report);

****Macro invocation: example 2: 
	Simulation number: 1000
	Sample size: 100
	Two arms, and randomization ratio: 1 vs. 1
	Proportion of Subgroups: 0.5 vs. 0.5
	Survival data distribution assumption: Weibull
	Parameter A in Weibull distribution: 2
	Parameter B in Weibull distribution: 2
	Ratio of Hazard Rate lowest vs. highest in the treatment period: 0.5 
	Censor Proportion: 0.2
	Subgroup 1 in arm 2 hazard rate changing type: Convex
	Subgroup 2 in arm 2 hazard rate changing type: Concave
	Weight functions for log-rank test: logrank fleming(0,1) wilcoxon
;

%PharmaSUG_SA_enrich
(randam_seed  = 2046, 
sim_number    = 1000, 
sample_size   = 100, 
subgrp1_prop  = 0.5, 
distribution  = "Weibull", 
exp_hr_h      = ,  
exp_hr_l      = , 
weibull_a     = 2, 
weibull_b     = 2, 
wb_hr_ratio   = 0.5, 
censor_prop   = 0.2, 
subgrp1_type  = "Convex", 
subgrp2_type  = "Concave",
path          = C:\Users\Chwzhang2015\Desktop\PharmaSUG 2019 June 15-20\!!!Code & dataset\PharmaSUG,
final_ds      = wei_2, 
weight        = logrank fleming(0,1) wilcoxon,
power_smry_ds = wei_smry_2,
report_name   = wei_smry_2_report);
